@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,10 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <gmime/gmime.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -30,12 +34,6 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#include <glib.h>
-#include <glib/gstdio.h>
-
-#include <gmime/gmime.h>
-
-
 static char *
 basename (char *path)
 {
@@ -107,11 +105,10 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 		GMimeObject *subpart;
 		int i, n;
 		
-		n = g_mime_multipart_get_number (multipart);
+		n = g_mime_multipart_get_count (multipart);
 		for (i = 0; i < n; i++) {
 			subpart = g_mime_multipart_get_part (multipart, i);
 			write_part_bodystructure (subpart, fp);
-			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
 		GMimeMessage *message;
@@ -299,12 +296,11 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 		id = g_stpcpy (buf, spec);
 		*id++ = '.';
 		
-		n = g_mime_multipart_get_number (multipart);
+		n = g_mime_multipart_get_count (multipart);
 		for (i = 0; i < n; i++) {
 			subpart = g_mime_multipart_get_part (multipart, i);
 			sprintf (id, "%d", i + 1);
 			write_part (subpart, uid, buf);
-			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
 		buf = g_strdup_printf ("%s/%s.TEXT", uid, spec);
@@ -323,9 +319,7 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 		
 		ostream = g_mime_stream_file_new (fp);
 		istream = g_mime_data_wrapper_get_stream (GMIME_PART (part)->content);
-		
 		g_mime_stream_write_to_stream (istream, ostream);
-		g_object_unref (istream);
 		g_object_unref (ostream);
 	}
 }
@@ -557,7 +551,7 @@ bodystruct_part_decode (unsigned char **in, unsigned char *inend)
 	part->envelope = NULL;
 	part->subparts = NULL;
 	
-	if (!strcasecmp (part->content.type, "multipart")) {
+	if (!g_ascii_strcasecmp (part->content.type, "multipart")) {
 		list = NULL;
 		tail = (struct _bodystruct *) &list;
 		
@@ -573,7 +567,7 @@ bodystruct_part_decode (unsigned char **in, unsigned char *inend)
 		}
 		
 		part->subparts = list;
-	} else if (!strcasecmp (part->content.type, "message") && !strcasecmp (part->content.subtype, "rfc822")) {
+	} else if (!g_ascii_strcasecmp (part->content.type, "message") && !g_ascii_strcasecmp (part->content.subtype, "rfc822")) {
 		part->envelope = decode_envelope (&inptr, inend);
 		part->subparts = bodystruct_part_decode (&inptr, inend);
 	} else {
@@ -621,13 +615,13 @@ bodystruct_dump (struct _bodystruct *part, int depth)
 	
 	fputc ('\n', stderr);
 	
-	if (!strcasecmp (part->content.type, "multipart")) {
+	if (!g_ascii_strcasecmp (part->content.type, "multipart")) {
 		part = part->subparts;
 		while (part != NULL) {
 			bodystruct_dump (part, depth + 1);
 			part = part->next;
 		}
-	} else if (!strcasecmp (part->content.type, "message") && !strcasecmp (part->content.subtype, "rfc822")) {
+	} else if (!g_ascii_strcasecmp (part->content.type, "message") && !g_ascii_strcasecmp (part->content.subtype, "rfc822")) {
 		depth++;
 		
 		for (i = 0; i < depth; i++)
@@ -737,7 +731,7 @@ reconstruct_part_content (GMimePart *part, const char *uid, const char *spec)
 	int fd;
 	
 	filename = g_strdup_printf ("%s/%s.TEXT", uid, spec);
-	if ((fd = open (filename, O_RDONLY))) {
+	if ((fd = open (filename, O_RDONLY, 0)) == -1) {
 		g_free (filename);
 		return;
 	}
@@ -755,16 +749,15 @@ reconstruct_part_content (GMimePart *part, const char *uid, const char *spec)
 static void
 reconstruct_message_part (GMimeMessagePart *msgpart, const char *uid, const char *spec)
 {
+	GMimeMessage *message;
 	GMimeParser *parser;
 	GMimeStream *stream;
 	char *filename;
 	int fd;
 	
-	if (msgpart->message)
-		g_object_unref (msgpart->message);
-	
 	filename = g_strdup_printf ("%s/%s.TEXT", uid, spec);
-	if ((fd = open (filename, O_RDONLY))) {
+	if ((fd = open (filename, O_RDONLY, 0))) {
+		g_mime_message_part_set_message (msgpart, NULL);
 		g_free (filename);
 		return;
 	}
@@ -776,8 +769,11 @@ reconstruct_message_part (GMimeMessagePart *msgpart, const char *uid, const char
 	g_mime_parser_set_scan_from (parser, FALSE);
 	g_object_unref (stream);
 	
-	msgpart->message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser);
 	g_object_unref (parser);
+	
+	g_mime_message_part_set_message (msgpart, message);
+	g_object_unref (message);
 }
 
 static void
@@ -816,7 +812,7 @@ reconstruct_multipart (GMimeMultipart *multipart, struct _bodystruct *body,
 		   manually rather than to do uber-fancy stuff */
 		
 		filename = g_strdup_printf ("%s/%s.HEADER", uid, subspec);
-		if ((fd = open (filename, O_RDONLY)) == -1) {
+		if ((fd = open (filename, O_RDONLY, 0)) == -1) {
 			g_free (filename);
 			return;
 		}
@@ -839,7 +835,7 @@ reconstruct_multipart (GMimeMultipart *multipart, struct _bodystruct *body,
 			reconstruct_part_content ((GMimePart *) subpart, uid, subspec);
 		}
 		
-		g_mime_multipart_add_part (multipart, subpart);
+		g_mime_multipart_add (multipart, subpart);
 		g_object_unref (subpart);
 		
 		part = part->next;
@@ -856,7 +852,7 @@ reconstruct_message (const char *uid)
 	int fd;
 	
 	filename = g_strdup_printf ("%s/HEADER", uid);
-	if ((fd = open (filename, O_RDONLY)) == -1) {
+	if ((fd = open (filename, O_RDONLY, 0)) == -1) {
 		g_free (filename);
 		return;
 	}
@@ -879,7 +875,7 @@ reconstruct_message (const char *uid)
 		GMimeStream *mem;
 		
 		filename = g_strdup_printf ("%s/BODYSTRUCTURE", uid);
-		if ((fd = open (filename, O_RDONLY)) == -1) {
+		if ((fd = open (filename, O_RDONLY, 0)) == -1) {
 			g_free (filename);
 			return;
 		}
@@ -937,7 +933,7 @@ int main (int argc, char **argv)
 		i++;
 	}
 	
-	if ((fd = open (argv[i], O_RDONLY)) == -1)
+	if ((fd = open (argv[i], O_RDONLY, 0)) == -1)
 		return 0;
 	
 	stream = g_mime_stream_fs_new (fd);
