@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -27,14 +27,13 @@
 G_BEGIN_DECLS
 
 extern GQuark gmime_error_quark;
-#define GMIME_ERROR_QUARK gmime_error_quark
 
 /**
  * GMIME_ERROR:
  *
  * The GMime error domain GQuark value.
  **/
-#define GMIME_ERROR GMIME_ERROR_QUARK
+#define GMIME_ERROR gmime_error_quark
 
 /**
  * GMIME_ERROR_IS_SYSTEM:
@@ -57,6 +56,16 @@ enum {
 	GMIME_ERROR_NO_VALID_RECIPIENTS = -5
 };
 
+
+extern GQuark gmime_gpgme_error_quark;
+
+/**
+ * GMIME_GPGME_ERROR:
+ *
+ * The GMime GpgMe error domain GQuark value.
+ **/
+#define GMIME_GPGME_ERROR gmime_gpgme_error_quark
+
 G_END_DECLS
 
 #endif /* __GMIME_ERROR_H__ */
