@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,11 +29,14 @@
 
 #include "gmime-message.h"
 #include "gmime-multipart.h"
+#include "gmime-multipart-signed.h"
+#include "gmime-multipart-encrypted.h"
 #include "gmime-part.h"
 #include "gmime-utils.h"
 #include "gmime-stream-mem.h"
 #include "gmime-table-private.h"
 #include "gmime-parse-utils.h"
+#include "gmime-events.h"
 
 
 /**
@@ -45,14 +48,7 @@
  * A #GMimeMessage represents an rfc822 message.
  **/
 
-
-typedef void (* EventCallback) (gpointer sender, gpointer args);
-
-void _internet_address_list_block_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-void _internet_address_list_unblock_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-void _internet_address_list_add_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-void _internet_address_list_remove_event_handler (InternetAddressList *list, EventCallback callback, gpointer user_data);
-
+extern GMimeEvent *_g_mime_header_list_get_changed_event (GMimeHeaderList *headers);
 
 static void g_mime_message_class_init (GMimeMessageClass *klass);
 static void g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass);
@@ -66,6 +62,7 @@ static const char *message_get_header (GMimeObject *object, const char *header);
 static gboolean message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void message_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 static ssize_t write_structured (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
@@ -73,20 +70,21 @@ static ssize_t write_received (GMimeStream *stream, const char *name, const char
 static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_msgid (GMimeStream *stream, const char *name, const char *value);
 
-static void to_list_changed (InternetAddressList *list, GMimeMessage *message);
-static void cc_list_changed (InternetAddressList *list, GMimeMessage *message);
-static void bcc_list_changed (InternetAddressList *list, GMimeMessage *message);
+static void to_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
+static void cc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
+static void bcc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message);
 
 
 static GMimeObjectClass *parent_class = NULL;
 
+
 static struct {
 	const char *name;
-	EventCallback changed_cb;
+	GMimeEventCallback changed_cb;
 } recipient_types[] = {
-	{ "To",  (EventCallback) to_list_changed  },
-	{ "Cc",  (EventCallback) cc_list_changed  },
-	{ "Bcc", (EventCallback) bcc_list_changed }
+	{ "To",  (GMimeEventCallback) to_list_changed  },
+	{ "Cc",  (GMimeEventCallback) cc_list_changed  },
+	{ "Bcc", (GMimeEventCallback) bcc_list_changed }
 };
 
 #define N_RECIPIENT_TYPES G_N_ELEMENTS (recipient_types)
@@ -146,6 +144,14 @@ g_mime_message_class_init (GMimeMessageClass *klass)
 	object_class->get_header = message_get_header;
 	object_class->get_headers = message_get_headers;
 	object_class->write_to_stream = message_write_to_stream;
+	object_class->encode = message_encode;
+}
+
+static void
+mime_part_headers_changed (GMimeHeaderList *headers, gpointer args, GMimeMessage *message)
+{
+	/* clear the message's header stream */
+	g_mime_header_list_set_stream (((GMimeObject *) message)->headers, NULL);
 }
 
 static void
@@ -155,7 +161,7 @@ connect_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_add_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_add (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -165,7 +171,7 @@ disconnect_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_remove_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_remove (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -175,7 +181,7 @@ block_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_block_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_block (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
@@ -185,14 +191,13 @@ unblock_changed_event (GMimeMessage *message, GMimeRecipientType type)
 	
 	list = message->recipients[type];
 	
-	_internet_address_list_block_event_handler (list, recipient_types[type].changed_cb, message);
+	g_mime_event_unblock (list->priv, recipient_types[type].changed_cb, message);
 }
 
 static void
 g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 {
 	GMimeHeaderList *headers = ((GMimeObject *) message)->headers;
-	InternetAddressList *list;
 	guint i;
 	
 	message->from = NULL;
@@ -232,6 +237,7 @@ static void
 g_mime_message_finalize (GObject *object)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
+	GMimeEvent *changed;
 	guint i;
 	
 	g_free (message->from);
@@ -250,8 +256,11 @@ g_mime_message_finalize (GObject *object)
 	g_free (message->message_id);
 	
 	/* unref child mime part */
-	if (message->mime_part)
+	if (message->mime_part) {
+		changed = _g_mime_header_list_get_changed_event (message->mime_part->headers);
+		g_mime_event_remove (changed, (GMimeEventCallback) mime_part_headers_changed, message);
 		g_object_unref (message->mime_part);
+	}
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -572,7 +581,7 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 			g_string_append (str, "\n\t");
 			len = 1;
 		} else if (lwsp) {
-			g_string_append_len (str, lwsp, part->start - lwsp);
+			g_string_append_len (str, lwsp, (size_t) (part->start - lwsp));
 		}
 		
 		g_string_append_len (str, part->start, part->len);
@@ -662,6 +671,7 @@ enum {
 	HEADER_SUBJECT,
 	HEADER_DATE,
 	HEADER_MESSAGE_ID,
+	HEADER_MIME_VERSION,
 	HEADER_UNKNOWN
 };
 
@@ -674,6 +684,7 @@ static const char *message_headers[] = {
 	"Subject",
 	"Date",
 	"Message-Id",
+	"MIME-Version",
 };
 
 enum {
@@ -707,9 +718,9 @@ process_header (GMimeObject *object, int action, const char *header, const char
 {
 	GMimeMessage *message = (GMimeMessage *) object;	
 	InternetAddressList *addrlist;
-	int offset, i;
 	time_t date;
-	gulong id;
+	int offset;
+	guint i;
 	
 	for (i = 0; i < G_N_ELEMENTS (message_headers); i++) {
 		if (!g_ascii_strcasecmp (message_headers[i], header))
@@ -765,6 +776,8 @@ process_header (GMimeObject *object, int action, const char *header, const char
 		g_free (message->message_id);
 		message->message_id = g_mime_utils_decode_message_id (value);
 		break;
+	case HEADER_MIME_VERSION:
+		break;
 	default:
 		return FALSE;
 	}
@@ -777,9 +790,6 @@ message_prepend_header (GMimeObject *object, const char *header, const char *val
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -791,7 +801,7 @@ message_prepend_header (GMimeObject *object, const char *header, const char *val
 		g_mime_header_list_prepend (object->headers, header, value);
 	
 	if (message->mime_part)
-		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 static void
@@ -799,9 +809,6 @@ message_append_header (GMimeObject *object, const char *header, const char *valu
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -813,7 +820,7 @@ message_append_header (GMimeObject *object, const char *header, const char *valu
 		g_mime_header_list_append (object->headers, header, value);
 	
 	if (message->mime_part)
-		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 static void
@@ -821,9 +828,6 @@ message_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
 	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
-	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
 	if (!g_ascii_strncasecmp ("Content-", header, 8))
@@ -835,25 +839,28 @@ message_set_header (GMimeObject *object, const char *header, const char *value)
 		g_mime_header_list_set (object->headers, header, value);
 	
 	if (message->mime_part)
-		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 }
 
 static const char *
 message_get_header (GMimeObject *object, const char *header)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
-	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return "1.0";
+	const char *value;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
-	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
-		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
-	else if (message->mime_part)
+	if (g_ascii_strncasecmp ("Content-", header, 8) != 0) {
+		if ((value = GMIME_OBJECT_CLASS (parent_class)->get_header (object, header)))
+			return value;
+		
+		if (!g_ascii_strcasecmp ("MIME-Version", header))
+			return "1.0";
+	} else if (message->mime_part) {
 		return g_mime_object_get_header (message->mime_part, header);
-	else
-		return NULL;
+	}
+	
+	return NULL;
 }
 
 static gboolean
@@ -862,10 +869,7 @@ message_remove_header (GMimeObject *object, const char *header)
 	GMimeMessage *message = (GMimeMessage *) object;
 	InternetAddressList *addrlist;
 	GMimeRecipientType type;
-	int i;
-	
-	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return FALSE;
+	guint i;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
@@ -928,7 +932,7 @@ message_remove_header (GMimeObject *object, const char *header)
 	}
 	
 	if (message->mime_part)
-		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 	
 	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
@@ -946,13 +950,14 @@ message_get_headers (GMimeObject *object)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), ba);
 	
-	if (message->mime_part && g_mime_header_list_has_raw (message->mime_part->headers)) {
+	if (message->mime_part && g_mime_header_list_get_stream (message->mime_part->headers)) {
 		/* if the mime part has raw headers, then it contains the message headers as well */
 		g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 	} else {
 		g_mime_header_list_write_to_stream (object->headers, stream);
 		if (message->mime_part) {
-			if (g_mime_object_get_header (message->mime_part, "Content-Type"))
+			if (g_mime_object_get_header (message->mime_part, "Content-Type") &&
+			    !g_mime_header_list_get (object->headers, "MIME-Version"))
 				g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
 			g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 		}
@@ -973,16 +978,16 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	ssize_t nwritten, total = 0;
 	
 	if (message->mime_part) {
-		if (!g_mime_header_list_has_raw (message->mime_part->headers)) {
-			/* if the mime part has raw headers, then it contains the message
-			 * headers as well otherwise it doesn't, so write them now */
+		if (!g_mime_header_list_get_stream (message->mime_part->headers)) {
 			if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
 				return -1;
 			
 			total += nwritten;
 			
-			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
-				return -1;
+			if (!g_mime_header_list_get (object->headers, "MIME-Version")) {
+				if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
+					return -1;
+			}
 			
 			total += nwritten;
 		}
@@ -1004,9 +1009,19 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	return total;
 }
 
+static void
+message_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	GMimeMessage *message = (GMimeMessage *) object;
+	
+	if (message->mime_part != NULL)
+		g_mime_object_encode (message->mime_part, constraint);
+}
+
+
 
 /**
- * g_mime_message_new: Create a new MIME Message object
+ * g_mime_message_new:
  * @pretty_headers: make pretty headers 
  *
  * If @pretty_headers is %TRUE, then the standard rfc822 headers are
@@ -1014,14 +1029,14 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * strictly a cosmetic thing, so if you are unsure, it is safe to say
  * no (%FALSE).
  *
- * Returns: an empty MIME Message object.
+ * Returns: an empty #GMimeMessage object.
  **/
 GMimeMessage *
 g_mime_message_new (gboolean pretty_headers)
 {
 	GMimeHeaderList *headers;
 	GMimeMessage *message;
-	int i;
+	guint i;
 	
 	message = g_object_newv (GMIME_TYPE_MESSAGE, 0, NULL);
 	
@@ -1142,19 +1157,19 @@ sync_recipient_header (GMimeMessage *message, GMimeRecipientType type)
 }
 
 static void
-to_list_changed (InternetAddressList *list, GMimeMessage *message)
+to_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message)
 {
 	sync_recipient_header (message, GMIME_RECIPIENT_TYPE_TO);
 }
 
 static void
-cc_list_changed (InternetAddressList *list, GMimeMessage *message)
+cc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message)
 {
 	sync_recipient_header (message, GMIME_RECIPIENT_TYPE_CC);
 }
 
 static void
-bcc_list_changed (InternetAddressList *list, GMimeMessage *message)
+bcc_list_changed (InternetAddressList *list, gpointer args, GMimeMessage *message)
 {
 	sync_recipient_header (message, GMIME_RECIPIENT_TYPE_BCC);
 }
@@ -1441,17 +1456,31 @@ g_mime_message_get_mime_part (GMimeMessage *message)
 void
 g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 {
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (GMIME_IS_OBJECT (mime_part));
+	GMimeEvent *changed;
 	
-	g_object_ref (mime_part);
-	g_mime_header_list_set_raw (mime_part->headers, NULL);
+	g_return_if_fail (mime_part == NULL || GMIME_IS_OBJECT (mime_part));
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	
+	if (message->mime_part == mime_part)
+		return;
 	
 	if (message->mime_part) {
-		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+		changed = _g_mime_header_list_get_changed_event (message->mime_part->headers);
+		g_mime_event_remove (changed, (GMimeEventCallback) mime_part_headers_changed, message);
+		
+		g_mime_header_list_set_stream (message->mime_part->headers, NULL);
 		g_object_unref (message->mime_part);
 	}
 	
+	if (mime_part) {
+		changed = _g_mime_header_list_get_changed_event (mime_part->headers);
+		g_mime_header_list_set_stream (mime_part->headers, NULL);
+		g_mime_event_add (changed, (GMimeEventCallback) mime_part_headers_changed, message);
+		g_object_ref (mime_part);
+	}
+	
+	g_mime_header_list_set_stream (((GMimeObject *) message)->headers, NULL);
+	
 	message->mime_part = mime_part;
 }
 
@@ -1475,3 +1504,98 @@ g_mime_message_foreach (GMimeMessage *message, GMimeObjectForeachFunc callback,
 	if (GMIME_IS_MULTIPART (message->mime_part))
 		g_mime_multipart_foreach ((GMimeMultipart *) message->mime_part, callback, user_data);
 }
+
+static gboolean
+part_is_textual (GMimeObject *mime_part)
+{
+	GMimeContentType *type;
+	
+	type = g_mime_object_get_content_type (mime_part);
+	
+	return g_mime_content_type_is_type (type, "text", "*");
+}
+
+static GMimeObject *
+multipart_guess_body (GMimeMultipart *multipart)
+{
+	GMimeContentType *type;
+	GMimeObject *mime_part;
+	int count, i;
+	
+	if (GMIME_IS_MULTIPART_ENCRYPTED (multipart)) {
+		/* check if this part has already been decrypted... */
+		if (!(mime_part = ((GMimeMultipartEncrypted *) multipart)->decrypted)) {
+			/* nothing more we can do */
+			return (GMimeObject *) multipart;
+		}
+		
+		if (GMIME_IS_MULTIPART (mime_part))
+			return multipart_guess_body ((GMimeMultipart *) mime_part);
+		else if (GMIME_IS_PART (mime_part) && part_is_textual (mime_part))
+			return mime_part;
+		else
+			return NULL;
+	}
+	
+	type = g_mime_object_get_content_type ((GMimeObject *) multipart);
+	if (g_mime_content_type_is_type (type, "multipart", "alternative")) {
+		/* very likely that this is the body - leave it up to
+		 * our caller to decide which format of the body it
+		 * wants to use. */
+		return (GMimeObject *) multipart;
+	}
+	
+	count = g_mime_multipart_get_count (multipart);
+	
+	if (!GMIME_IS_MULTIPART_SIGNED (mime_part)) {
+		/* if the body is in here, it has to be the first part */
+		count = 1;
+	}
+	
+	for (i = 0; i < count; i++) {
+		mime_part = g_mime_multipart_get_part (multipart, i);
+		
+		if (GMIME_IS_MULTIPART (mime_part)) {
+			if ((mime_part = multipart_guess_body ((GMimeMultipart *) mime_part)))
+				return mime_part;
+		} else if (GMIME_IS_PART (mime_part)) {
+			if (part_is_textual (mime_part))
+				return mime_part;
+		}
+	}
+	
+	return NULL;
+}
+
+
+/**
+ * g_mime_message_get_body:
+ * @message: MIME Message
+ *
+ * Attempts to identify the MIME part containing the body of the
+ * message.
+ *
+ * Returns: a #GMimeObject containing the textual content that appears
+ * to be the main body of the message.
+ *
+ * Note: This function is NOT guarenteed to always work as it
+ * makes some assumptions that are not necessarily true. It is
+ * recommended that you traverse the MIME structure yourself.
+ **/
+GMimeObject *
+g_mime_message_get_body (GMimeMessage *message)
+{
+	GMimeObject *mime_part;
+	
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	
+	if (!(mime_part = message->mime_part))
+		return NULL;
+	
+	if (GMIME_IS_MULTIPART (mime_part))
+		return multipart_guess_body ((GMimeMultipart *) mime_part);
+	else if (GMIME_IS_PART (mime_part) && part_is_textual (mime_part))
+		return mime_part;
+	
+	return NULL;
+}
