@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -27,9 +27,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 #include <fcntl.h>
 #include <errno.h>
 #include <time.h>
@@ -65,6 +63,7 @@ static gboolean multipart_remove_header (GMimeObject *object, const char *header
 static void multipart_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *multipart_get_headers (GMimeObject *object);
 static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void multipart_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 /* GMimeMultipart class methods */
 static void multipart_clear (GMimeMultipart *multipart);
@@ -126,6 +125,7 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 	object_class->set_content_type = multipart_set_content_type;
 	object_class->get_headers = multipart_get_headers;
 	object_class->write_to_stream = multipart_write_to_stream;
+	object_class->encode = multipart_encode;
 	
 	klass->add = multipart_add;
 	klass->clear = multipart_clear;
@@ -304,6 +304,19 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	return total;
 }
 
+static void
+multipart_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	GMimeMultipart *multipart = (GMimeMultipart *) object;
+	GMimeObject *subpart;
+	int i;
+	
+	for (i = 0; i < g_mime_multipart_get_count (multipart); i++) {
+		subpart = g_mime_multipart_get_part (multipart, i);
+		g_mime_object_encode (subpart, constraint);
+	}
+}
+
 
 /**
  * g_mime_multipart_new:
@@ -482,18 +495,20 @@ ptr_array_insert (GPtrArray *array, guint index, gpointer object)
 	unsigned char *dest, *src;
 	guint n;
 	
-	g_ptr_array_set_size (array, array->len + 1);
-	
-	if (index == array->len) {
-		/* need to move items down */
+	if (index < array->len) {
+		/* need to shift some items */
+		g_ptr_array_set_size (array, array->len + 1);
+		
 		dest = ((unsigned char *) array->pdata) + (sizeof (void *) * (index + 1));
 		src = ((unsigned char *) array->pdata) + (sizeof (void *) * index);
 		n = array->len - index - 1;
 		
 		g_memmove (dest, src, (sizeof (void *) * n));
+		array->pdata[index] = object;
+	} else {
+		/* the easy case */
+		g_ptr_array_add (array, object);
 	}
-	
-	array->pdata[index] = object;
 }
 
 static void
@@ -718,7 +733,7 @@ g_mime_multipart_get_count (GMimeMultipart *multipart)
 static void
 read_random_pool (unsigned char *buffer, size_t bytes)
 {
-#ifdef HAVE_UNISTD_H
+#ifdef __unix__
 	size_t nread = 0;
 	ssize_t n;
 	int fd;
