@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -24,6 +24,7 @@
 #endif
 
 #include <string.h>
+#include <errno.h>
 
 #include "gmime-stream-mem.h"
 
@@ -51,7 +52,7 @@ static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
 static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
 static gint64 stream_tell (GMimeStream *stream);
-static ssize_t stream_length (GMimeStream *stream);
+static gint64 stream_length (GMimeStream *stream);
 static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
@@ -131,16 +132,19 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	gint64 bound_end;
 	ssize_t n;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	bound_end = stream->bound_end != -1 ? stream->bound_end : (gint64) mem->buffer->len;
 	
-	n = MIN (bound_end - stream->position, (gint64) len);
+	n = (size_t) MIN (bound_end - stream->position, (gint64) len);
 	if (n > 0) {
 		memcpy (buf, mem->buffer->data + stream->position, n);
 		stream->position += n;
 	} else if (n < 0) {
-		/* set errno?? */
+		errno = EINVAL;
 		n = -1;
 	}
 	
@@ -154,20 +158,23 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	gint64 bound_end;
 	ssize_t n;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	if (stream->bound_end == -1 && stream->position + len > mem->buffer->len) {
-		g_byte_array_set_size (mem->buffer, stream->position + len);
+		g_byte_array_set_size (mem->buffer, (guint) stream->position + len);
 		bound_end = mem->buffer->len;
 	} else
 		bound_end = stream->bound_end;
 	
-	n = MIN (bound_end - stream->position, (gint64) len);
+	n = (size_t) MIN (bound_end - stream->position, (gint64) len);
 	if (n > 0) {
 		memcpy (mem->buffer->data + stream->position, buf, n);
 		stream->position += n;
 	} else if (n < 0) {
-		/* FIXME: set errno?? */
+		errno = EINVAL;
 		n = -1;
 	}
 	
@@ -177,7 +184,13 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 static int
 stream_flush (GMimeStream *stream)
 {
-	/* no-op */
+	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
+	
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
+	
 	return 0;
 }
 
@@ -190,7 +203,6 @@ stream_close (GMimeStream *stream)
 		g_byte_array_free (mem->buffer, TRUE);
 	
 	mem->buffer = NULL;
-	stream->position = 0;
 	
 	return 0;
 }
@@ -201,7 +213,8 @@ stream_eos (GMimeStream *stream)
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	gint64 bound_end;
 	
-	g_return_val_if_fail (mem->buffer != NULL, TRUE);
+	if (mem->buffer == NULL)
+		return TRUE;
 	
 	bound_end = stream->bound_end != -1 ? stream->bound_end : (gint64) mem->buffer->len;
 	
@@ -213,7 +226,12 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	
-	return mem->buffer ? 0 : -1;
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
+	
+	return 0;
 }
 
 static gint64
@@ -222,7 +240,10 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	gint64 bound_end, real = stream->position;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	bound_end = stream->bound_end != -1 ? stream->bound_end : (gint64) mem->buffer->len;
 	
@@ -238,10 +259,24 @@ stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 		break;
 	}
 	
-	if (real < stream->bound_start)
-		real = stream->bound_start;
-	else if (real > bound_end)
-		real = bound_end;
+	if (real < stream->bound_start) {
+		errno = EINVAL;
+		return -1;
+	}
+	
+	if (stream->bound_end != -1 && real > bound_end) {
+		errno = EINVAL;
+		return -1;
+	}
+	
+	if (real > bound_end) {
+		if (real > G_MAXUINT) {
+			errno = ENOSPC;
+			return -1;
+		}
+		
+		g_byte_array_set_size (mem->buffer, (guint) real);
+	}
 	
 	stream->position = real;
 	
@@ -253,18 +288,24 @@ stream_tell (GMimeStream *stream)
 {
 	GMimeStreamMem *mem = (GMimeStreamMem *) stream;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	return stream->position;
 }
 
-static ssize_t
+static gint64
 stream_length (GMimeStream *stream)
 {
 	GMimeStreamMem *mem = GMIME_STREAM_MEM (stream);
 	gint64 bound_end;
 	
-	g_return_val_if_fail (mem->buffer != NULL, -1);
+	if (mem->buffer == NULL) {
+		errno = EBADF;
+		return -1;
+	}
 	
 	bound_end = stream->bound_end != -1 ? stream->bound_end : (gint64) mem->buffer->len;
 	
@@ -288,7 +329,7 @@ stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 /**
  * g_mime_stream_mem_new:
  *
- * Creates a new GMimeStreamMem object.
+ * Creates a new #GMimeStreamMem object.
  *
  * Returns: a new memory stream.
  **/
@@ -310,7 +351,7 @@ g_mime_stream_mem_new (void)
  * g_mime_stream_mem_new_with_byte_array:
  * @array: source data
  *
- * Creates a new GMimeStreamMem with data @array.
+ * Creates a new #GMimeStreamMem with data @array.
  *
  * Returns: a new memory stream using @array.
  **/
@@ -331,9 +372,9 @@ g_mime_stream_mem_new_with_byte_array (GByteArray *array)
 /**
  * g_mime_stream_mem_new_with_buffer:
  * @buffer: stream data
- * @len: data length
+ * @len: buffer length
  *
- * Creates a new GMimeStreamMem object and initializes the stream
+ * Creates a new #GMimeStreamMem object and initializes the stream
  * contents with the first @len bytes of @buffer.
  *
  * Returns: a new memory stream initialized with @buffer.
@@ -356,7 +397,7 @@ g_mime_stream_mem_new_with_buffer (const char *buffer, size_t len)
 
 /**
  * g_mime_stream_mem_get_byte_array:
- * @mem: memory stream
+ * @mem: a #GMimeStreamMem
  *
  * Gets the byte array from the memory stream.
  *
@@ -373,13 +414,13 @@ g_mime_stream_mem_get_byte_array (GMimeStreamMem *mem)
 
 /**
  * g_mime_stream_mem_set_byte_array:
- * @mem: memory stream
+ * @mem: a #GMimeStreamMem
  * @array: stream data
  *
  * Sets the byte array on the memory stream.
  *
  * Note: The memory stream is not responsible for freeing the byte
- * array.
+ * array. Use g_mime_stream_mem_set_owner() to change this behavior.
  **/
 void
 g_mime_stream_mem_set_byte_array (GMimeStreamMem *mem, GByteArray *array)
@@ -405,7 +446,7 @@ g_mime_stream_mem_set_byte_array (GMimeStreamMem *mem, GByteArray *array)
 
 /**
  * g_mime_stream_mem_get_owner:
- * @mem: memory stream
+ * @mem: a #GMimeStreamMem
  *
  * Gets whether or not @mem owns the backend memory buffer.
  *
@@ -423,8 +464,8 @@ g_mime_stream_mem_get_owner (GMimeStreamMem *mem)
 
 /**
  * g_mime_stream_mem_set_owner:
- * @mem: memory stream
- * @owner: owner
+ * @mem: a #GMimeStreamMem
+ * @owner: %TRUE if this stream should own the #GByteArray or %FALSE otherwise
  *
  * Sets whether or not @mem owns the backend memory buffer.
  *
